#!/usr/bin/env python
# coding=latin-1
'''
Copyright (C) 2012: W.G.Vree

This program is free software; you can redistribute it and/or modify it under the terms of the
GNU General Public License as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details. <http://www.gnu.org/licenses/gpl.html>.
'''

import re, os, sys
from optparse import OptionParser

VERSION = 13

rd = re.compile (r'd="M([\d.]+) ([\d.]+)v-24.00"/>')    # x,y coors (bottom) of small measure bars (in top staff)
rg = re.compile (r'transform="translate\(([\d.]+),([\d.]+)\) scale\(1.2,1\)">') # x,y of time signature
rn = re.compile (r'y="-1".*>(\d+)</text>')              # denominator time signature
rt = re.compile (r'y="-13".*>(\d+)</text>')             # numerator time signature
ru = re.compile (r'x="([\d.]+)" y="([\d.]+)">= ([\d.]+)</text>')    # x-coor, y-coor and value of tempo (can be float!)
ry = re.compile (r'M([\d.]+) ([\d.]+)h([\d.]+)m-\3 -6h\3')  # left-x and y-coor of bottom line of current staff
rs = re.compile (r'<g transform="scale\(([\d.]+)\)"')   # scale value of a top svg group
rv = re.compile (r'width=".*" height=".*" viewBox="0 0 (\d+) (\d+)">')  # set the document size -> viewbox size
r1 = re.compile (r'<svg ')
r2 = re.compile (r'</svg>')

def init (): # reset the globals
    global tempo, msreDur, time, qtime, qbeats, musicOffset
    tempo = 120  # quarter notes per minute (default)
    msreDur = 4  # number of beats in a measure (default)
    time = 0.0   # play back time at the beginning of measure 1
    qtime = 0    # time in beats since start
    qbeats = []  # time in secs of the beats if optional file provided
    musicOffset = 0 # time from begin media until first beat of the score is heard

def orderElems (svg):
    elems = {}  # elements of each staff: elems [y-coor] = [(x-coor, type, value), ...]
    for line in svg:
        rp = rd.search (line)   # find small measure bar
        if rp:                  # small measure bar found (only appears in first staff of a system)
            x, y = rp.groups () # bottom coors of small bar == bottom of first staff
            if y not in elems:  # a new system (bottom y-coor not yet encountered)
                elems [y] = []  # start new element list for this staff
    staves, systems = [], []    # collect staves in a system
    for line in svg:
        rp = ry.search (line)   # look for a staff
        if rp:
            x, y, h = rp.groups ()      # left-x and y-coor of bottom line
            if y in elems and staves:   # top staff of a new system
                systems.append (staves) # store staves of previous system
                staves = []             # accumulate next system
            staves.append ((x, y))      # store x, y of all staves in a system
    if staves:                  # last system
        systems.append (staves) # store staves of previous system
    svgit = iter (svg)
    for line in svgit:           # collect elements in each top staff and put them in x-order
        rp = rs.search (line)    # search scale factor
        if rp:                   # all music is in one svg group with a scale factor
            scale = rp.group (1)
            continue
        rp = rg.search (line)    # find the time signature
        if rp:
            x, y = rp.groups ()  # x, y position of signature
            if y not in elems: continue    # not in a top staff
            line = svgit.next ()   # the denominator is on the next line
            rp = rn.search (line) 
            if not rp: raise Exception ('cannot find the denomintor of the time signature')
            denm = int (rp.group (1)) # denominator of the signature (not used in this version)
            line = svgit.next ()   # the numerator is on the next line
            rp = rt.search (line) 
            if not rp: raise Exception ('cannot find the numerator of the time signature')
            num =  int (rp.group (1)) # numerator of the time signature
            elems [y].append ((float (x), 'sig', num))
            continue
        rp = ru.search (line)    # look for tempo value
        if rp and not qbeats:    # only look for tempos if no external tempos are defined
            x, y, tp = rp.groups () # x-coor whence the tempo tp is valid
            offsets = [(float (ystr) - float (y), ystr) for ystr in elems]  # offset =  y_bottom staff - y_tempo
            offsets = sorted ([(dy, ystr) for dy, ystr in offsets if dy > 0 and dy < 80])   # find smallest positive offset
            ybot = offsets [0][1]   # string value of smallest positive offset == staff index in elems
            elems [ybot].append ((float (x), 'tem', float (tp)))
            continue
        rp = rd.search (line)    # find small measure bar
        if rp:                   # small measure bar found (only appears in top staff of a system)
            x,y = rp.groups ()   # bottom coors of small bar
            if y not in elems: continue    # not in a top staff
            elems [y].append ((float (x), 'bar', 0))
    for y in elems: elems[y].sort ()
    return scale, elems, systems

def parseSvg (svg_list):
    global tempo, msreDur, time, qtime
    ypos = []   # for each measure the y-coor of the bottom (line) of the top staff (in a system)
    rpos = []   # for each measure the x-coor of the right side bar
    lpos = []   # for each measure the x-coor of the left side bar
    times = []  # for each measure the play back time at the right bar
    staffh = [] # for each measure the height of the staff system (below bottom line of the top staff)
    tempos = [] # stack with (x-coor, tempo) values of future tempo markings
    staffs = [] # the y-coors of the bottom (line) of all staffs in the current system
    ystaff = 0  # the y-coor of the bottom (line) of the top staff in the current system
    xstaff = 0  # and the left x-coor of that staff

    scale, elems, systems = orderElems (svg_list)
    xs = zip (map (float , elems.keys ()), elems.keys ())
    staffkeys = [ystr for yfloat, ystr in sorted (xs)]
    for istf, stfk in enumerate (staffkeys):
        xstaff = systems [istf][0][0]       # [0][0] -> first staff, x-coor
        ystaff = systems [istf][0][1]       # [0][1] -> first staff, y-coor
        y1 = float (systems [istf][-1][1])  # [-1][1] -> last staff, y-coor
        sysheight = y1 - float (ystaff)     # height of current staff system
        first = 1
        for x, type, val in elems [stfk]:
            #~ print x, type, val
            if type == 'sig': msreDur = val # number of beats in a measure
            elif type == 'tem': tempo = val # tempo becomes valid
            elif type == 'bar':
                if rpos and abs (x - float (rpos[-1])) <= 10: # multiple bar lines close together
                    continue                # skip multiple lines (staff endings)
                if first:                   # first measure in this staff
                    lpos.append (xstaff)    # lpos = left side staff, 1st time in each staff system
                    first = 0
                else:
                    lpos.append (rpos[-1])  # lpos == previous rpos (2nd and further)
                ypos.append (ystaff)        # remember the y-coor of every small measure bar
                rpos.append ('%.2f' % x)    # also remember the x-coor (== time position)
                time += msreDur * 60 / tempo # play back time at the end of this bar
                if qbeats and qtime < len (qbeats): # external beat list available and long enough
                    time = qbeats [qtime]           # end time ot this measure (forget tempo)
                qtime += 1                  # measure number
                times.append ('%.2f' % time) # remember play back time for each measure
                staffh.append ('%.1f' % sysheight) # remember the height of the staff system
                continue                    # which is needed later to draw the music cursor
            else: raise Exception ('element type unknown')
    s1 = 'yposarr.push ([%s]);' % ','.join (ypos) # make javascript code to store all found values into
    s2 = 'rposarr.push ([%s]);' % ','.join (rpos) # javascript arrays. This code will be be prepended to 
    s3 = 'lposarr.push ([%s]);' % ','.join (lpos) # the rest of the javascript in the html5 page to be genetated
    s4 = 'timesarr.push ([%s]);' % ','.join (times)
    s5 = 'staffharr.push ([%s]);' % ','.join (staffh)
    s6 = 'scalearr.push (%s);' % scale
    return [s1,s2,s3,s4,s5,s6]

def readSvg (f): # just read  the <svg> ... </svg> tag and their content for one page of the music
    try:
        ro = None
        while not ro: # skip lines until the start tag: <svg
            xs = f.next()
            ro = r1.search (xs)
        ys = [xs]
        ro = None
        while not ro: # read and store lines until the end tag: </svg>
            xs = f.next()
            xs = rv.sub (r'width="\1" height="\2">', xs)    # document size to viewbox size (if a viewbox attr is present)
            ys.append (xs)
            ro = r2.search (xs)
        return ys # return all lines of the <svg> ... </svg> tag
    except:
        return []

def genHtml (padfnm, withVideo, noSync, odir, scrollMargin=100, videoHeight=40):
    global musicOffset
    init ()
    pad, fnmext = os.path.split (padfnm)
    fnm, ext = os.path.splitext (fnmext)
    bfnm = os.path.join (pad, fnm + '_beats.txt')
    if os.path.exists (bfnm):
        ft = open (bfnm)        # read timings from file if it exists
        x = ft.next ()
        while x.startswith ('#'): x = ft.next ();   # skip initial comments
        x = x.split ()[0]       # only take the first value on the line
        musicOffset = float (x) # time from begin media until first note of the score is heard
        print 'offset to the music: %.2f secs' % musicOffset
        for x in ft:            # read measure end times
            if not x.strip (): continue     # skip empty lines
            if x.startswith ('#'): continue # skip comments
            x = x.split ()[0]   # only the first value
            t = float (x)       # absolute time of the beat
            qbeats.append (t)   # list of end times
        ft.close ()
        print '%d external measure times found' % len (qbeats)
    svgs = []
    defs = ['var fnm = "%s";' % fnm, 'var noSync = %d;' % (noSync and 1 or 0),
            'var musicOffset = %s;' % musicOffset, 'var scrollMargin = %d;' % scrollMargin,
            'var scrollPage = 0;', 'var yposarr = [];', 'var rposarr = [];', 'var lposarr = [];',
            'var timesarr = [];', 'var staffharr = [];', 'var scalearr = [];']
    f = open (padfnm)
    svg = readSvg (f)
    while svg: # parse all music pages and store the computed arrays as javascript code
        svgs.append (svg)              # store music pages
        defs += parseSvg (svg)         # computed positions
        svg = readSvg (f)              # read next page
    f.close ()
    defs = '\n'.join (defs)            # statements for computed arrays all into one string

    # Write the generated js code with timing and location info 
    # to the heading section of a new html5 page
    if odir: pad = odir
    ofnm = os.path.join (pad, fnm + '_play.html') # the output file
    g = file (ofnm, 'w')     # same base name as the xhtml file with _play.html appended
    g.write ('''<!DOCTYPE html>
<html style="height:100%%"><head>
<meta http-equiv="content-type" content="text/html; charset=UTF-8">
<style>
    #seekdial { position:absolute; left:30%%; top:30%%; visibility:hidden;
                background-color:#bcd; font-size:large; padding:30px; z-index:100 }
    #help { position:absolute; top:100px; right:65px; width:12px; height:20px; padding-left:5px;
            overflow:hidden; z-index: 1000; background-color:#eee; border: 1px solid black;}
    #help:link, #help:visited { ; }
    #help:hover { width:500px; height:300px; right:65px; background-color:white; }
</style>
<script>
%s''' % defs)                   # music dependent js code with timing->location info

    # Now include the static player code (raw string!! for possible \n in code)
    g.write (r'''
var xmlns="http://www.w3.org/2000/svg";var mark,cur_page,svgs;var audiotag,gtijd,gdur,stat,knop,mnum,gevnt,seekdial;
var cur_measure;var page_mnums;var cur_mtime=0;var page_btime=0;var page_etime=0;var mnum_ij=[];var ij_mnum=[];
var x_offset=0;var y_offset=0;var m_height=0;var m_width=0;var y_scroll=0;var x_scroll=0;var msre_pad=6;
var staff_height=24;var scale;var first_timestamp;var ypos;var rpos;var lpos;var times;var staffh;var gDuration=0;
var gFac=0.1;var startmsre=0;var synarr=[];function page_btimes(a){if(a==0){return 0}timesprev=timesarr[a-1];
return timesprev[timesprev.length-1]}function prev(){if(cur_page==0){return}cur_page--;page(cur_page,1)
}function next(){if(cur_page==svgs.length-1){cur_page=0}else{cur_page++}page(cur_page,1)}function page(f,d){cur_page=f;
ypos=yposarr[f];rpos=rposarr[f];lpos=lposarr[f];times=timesarr[f];staffh=staffharr[f];scale=scalearr[f];
page_etime=times[times.length-1];page_btime=page_btimes(f);cur_measure=0;cur_mtime=page_btime;musicTime=audiotag.currentTime-musicOffset;
if(musicTime<0){musicTime=0}if(d&&(musicTime<page_btime||musicTime>=page_etime-0.000001)){audiotag.currentTime=page_btime+musicOffset
}for(var a=0;a<svgs.length;a++){var c=svgs[a];if(a==f){c.setAttribute("style","display:block")}else{c.setAttribute("style","display:none")
}if(document.activeElement!=mnum){mnum.value=cur_measure+page_mnums[f]+1}}}function getMusicOffset(){var a=document.getElementById("music");
x_offset=a.offsetLeft;y_offset=a.offsetTop;m_height=a.clientHeight;m_width=a.clientWidth;y_scroll=a.scrollTop;
x_scroll=a.scrollLeft}function setglobals(){audiotag=document.getElementById("sound");gtijd=document.getElementById("time");
gdur=document.getElementById("duur");stat=document.getElementById("status");knop=document.getElementById("playbtn");
mnum=document.getElementById("msrenum");svgs=document.getElementsByTagName("svg");gevnt=document.getElementById("event_display");
help=document.getElementById("help");syncBx=document.getElementById("sync");seekdial=document.getElementById("seekdial");
mark=document.getElementById("mark");page_mnums=[0];mnum_ij=[];ij_mnum=[];var c=0;for(var f=0;f<timesarr.length;
f++){var a=[];for(var d=0;d<timesarr[f].length;++d){mnum_ij.push([f,d]);a.push(c);c+=1}ij_mnum.push(a);
page_mnums.push(c)}u=window.location.href;parstr=u.split("?")[1];autostart=0;startmsre=0;if(parstr){pars=parstr.split("&");
for(f=0;f<pars.length;f++){if(pars[f]=="start"){autostart=1}else{startmsre=pars[f]}}}page(0,1);knop.disabled=0;
mnum.disabled=0;if(noSync){syncBx.setAttribute("style","display:none")}else{syncBx.disabled=0}document.getElementById("nextbtn").disabled=0;
document.getElementById("prevbtn").disabled=0;document.getElementById("music").style.display="block";
if(gDuration!=0){gdur.innerHTML="duration: "+gDuration.toFixed(1)+" secs";if(startmsre){mnum.value=startmsre;
gotoMsre()}}if(autostart){play()}getMusicOffset()}function klik(o){getMusicOffset();var k=(window.pageYOffset+o.clientY+y_scroll-y_offset)/scale;
var l=(o.clientX+x_scroll-x_offset)/scale;var h=page_btime;for(var f=0;f<ypos.length;++f){var a=times[f];
var c=ypos[f]+staffh[f];if((c>k)&&(rpos[f]>l)){var d=(l-lpos[f])/(rpos[f]-lpos[f])*(a-h);if(syncBx.checked&&knop.value=="pause"){syncTimes(f,d)
}else{var g=h+d;cur_measure=f;cur_mtime=h;audiotag.currentTime=g+musicOffset}break}h=a}}function klikChk(c){var a=document.getElementById("sync_out");
if(syncBx.checked){a.style.display="block";tick()}else{a.style.display="none"}}function tick(){getMusicOffset();
if(cur_page==svgs.length-1&&cur_measure>=times.length){return}gtijd.innerHTML="time: "+audiotag.currentTime.toFixed(1)+" secs";
tijd=audiotag.currentTime-musicOffset;if(tijd<0){tijd=0}if(tijd>times[cur_measure]){cur_mtime=times[cur_measure];
cur_measure++;if(cur_measure>=times.length){if(cur_page==svgs.length-1){stop();return}else{next()}}}var d=lpos[cur_measure]+msre_pad;
var c=rpos[cur_measure]-msre_pad;var o=ypos[cur_measure];var a=times[cur_measure];var l=staffh[cur_measure];
var r=(tijd-cur_mtime)/(a-cur_mtime);var p=d+(c-d)*r;mark.style.left=p*scale+"px";mark.style.top=(o-staff_height)*scale+"px";
mark.style.height=(l+staff_height)*scale+"px";var s=parseInt((o+l)*scale+scrollMargin);var g=parseInt(o*scale-scrollMargin);
var f=document.getElementById("music");if(g<y_scroll){f.scrollTop=g}if(s>m_height+y_scroll){f.scrollTop=g
}var k=parseInt(d*scale);var q=parseInt(c*scale);if(k<x_scroll){f.scrollLeft=k}if(q>m_width+x_scroll){f.scrollLeft=k
}if(scrollPage){y1=window.pageYOffset;y2=y1+window.innerHeight;yb=s-f.scrollTop+y_offset;yt=g-f.scrollTop+y_offset;
if(yt<y1){scrollTo(0,yt)}if(yb>y2){scrollTo(0,yb-window.innerHeight)}}if(document.activeElement!=mnum){mnum.value=cur_measure+page_mnums[cur_page]+1
}if(syncBx.checked){showSyncInfo(a-cur_mtime)}}function play(){if(knop.value=="play"){if(cur_page==svgs.length-1&&cur_measure>=times.length){page(0,1)
}audiotag.play();knop.value="pause"}else{stop()}}function stop(){audiotag.pause();knop.value="play"}function einde(){stop();
cur_page=svgs.length-1;cur_measure=times.length}function gotoMsre(){var c=mnum.value-1;if(isNaN(c)){c=0
}var f=mnum_ij.length;while(c<0){c=f+c}if(c>=f){c=f-1}var a,d;a=mnum_ij[c][0];d=mnum_ij[c][1];if((d==cur_measure)&&!startmsre){return
}if(d==0){new_time=page_btimes(a)}else{new_time=timesarr[a][d-1]}page(a,0);cur_measure=d;cur_mtime=new_time;
audiotag.currentTime=cur_mtime+musicOffset;mnum.blur();startmsre=0}function loadAudio(){if(audiotag==undefined){return
}nrange=audiotag.buffered.length;loaded=0;for(i=0;i<nrange;++i){b=audiotag.buffered.start(i);e=audiotag.buffered.end(i);
loaded+=e-b}stat.innerHTML="loaded: "+loaded.toFixed(1)+" secs"}function setDur(a){gDuration=a.target.duration;
if(gdur!=undefined){gdur.innerHTML="duration: "+gDuration.toFixed(1)+" secs"}if(startmsre){mnum.value=startmsre;
gotoMsre()}}function seekpos(a){seekdial.style.visibility="visible"}function seekedpos(a){seekdial.style.visibility="hidden"
}function showevent(a){if(first_timestamp==undefined){first_timestamp=a.timeStamp}secs=(a.timeStamp-first_timestamp)/1000000;
gevnt.innerHTML+=a.type+" event after "+secs.toFixed(2)+" secs<br>"}function showval(a){gevnt.innerHTML+=a+"<br>"
}function showSyncInfo(a){gevnt.innerHTML="duration measure:<br>"+a.toFixed(3)+" sec.<br>";gevnt.innerHTML+="offset first measure:<br>"+musicOffset.toFixed(3)+" sec."
}function restart(){page(0,0);cur_measure=0;cur_mtime=0;audiotag.currentTime=cur_mtime+musicOffset}function keyDown(a){key=a.keyCode?a.keyCode:a.which;
if(key==219||key==88){n=Number(mnum.value)-1;if(n>=1){mnum.value=n;gotoMsre()}}else{if(key==221||key==67){mnum.value=Number(mnum.value)+1;
gotoMsre()}else{if(key==80){play()}else{if(!syncBx.checked){return}}}}if(a.ctrlKey){changeOffset(gFac,key)
}else{if(key==190){changeTimes(+gFac,a.shiftKey)}else{if(key==188){changeTimes(-gFac,a.shiftKey)}else{if(key==82){musicOffset=0;
restart()}else{if(key==83){musicOffset=audiotag.currentTime;restart()}else{if(key==87){saveTiming()}else{return
}}}}}}a.preventDefault()}function changeTimes(a,c){msnm=cur_measure+page_mnums[cur_page];n=0;tbeg=0;tendprev=0;
for(i=0;i<timesarr.length;i++){for(j=0;j<timesarr[i].length;++j){tend=timesarr[i][j];if(n>=msnm){dur=tend-tbeg;
if(n==msnm){dur+=a}else{if(c){dur+=a}}tend=tendprev+dur}tbeg=timesarr[i][j];tendprev=tend;timesarr[i][j]=tend;
n+=1}}m=cur_measure;mt=cur_mtime;page(cur_page,0);cur_measure=m;cur_mtime=mt;audiotag.currentTime=cur_mtime+musicOffset;
tick()}function changeOffset(c,a){if(a==190){musicOffset+=c}else{if(a==188){musicOffset-=c}else{return
}}restart()}function syncTimes(g,d){var t=audiotag.currentTime-musicOffset-d;var r=g+page_mnums[cur_page];
if(r==0){musicOffset=t;restart();return}else{r-=1}var c=-1;var q=mnum_ij.length;var v=1;for(var o in synarr){var k=synarr[o];
if(k>c&&k<r){c=k}if(k<q&&k>r){q=k}if(k==r){v=0}}var o,l;var s=0;if(q<mnum_ij.length){o=mnum_ij[q][0];
l=mnum_ij[q][1];s=timesarr[o][l]}var h=0;if(c>-1){o=mnum_ij[c][0];l=mnum_ij[c][1];h=timesarr[o][l]}var p=(t-h)/(r-c);
var a=(s-t)/(q-r);if(s==0){a=p}var f=0;for(var f=0;f<mnum_ij.length;++f){o=mnum_ij[f][0];l=mnum_ij[f][1];
if(f>c&&f<=r){timesarr[o][l]=h+(f-c)*p}if(f>r&&f<q){timesarr[o][l]=t+(f-r)*a}}if(v){synarr.push(r)}if(r>=1){mnum.value=r
}startmsre=1;gotoMsre()}function saveTiming(){var d="########################################\n";d+='# This page contains your synchronization data. Save the page as "'+fnm+'_beats.txt"\n';
d+="# to include the timing data permanently in the (next) webpage you generate with mscweb.\n";d+="# Save the page as plain text file (not as html) and put it in the same directory as your xml file.\n";
d+='# Use the Save As command from the File menu of your browser and change the file type to "text"\n';
d+="########################################\n#\n";d+=musicOffset.toFixed(3)+" offset\n";for(i=0;i<timesarr.length;
i++){for(j=0;j<timesarr[i].length;++j){d+=timesarr[i][j].toFixed(3)+" "+(page_mnums[i]+j+1)+"\n"}}d+="#\n########################################\n";
if(/MSIE/.test(navigator.userAgent)){if(!confirm("Do you want to save your synchronization data?")){return
}document.open("text/html");document.write("<pre>"+d+"</pre>");document.close()}else{var c=document.createElement("a");
c.href="data:text/plain;charset=utf-8,"+encodeURIComponent(d);c.download=fnm+"_beats.txt";c.text="Save synchronization data";
document.getElementById("saveDiv").appendChild(c);c.click()}};
</script>
</head>''')

    # Next, write buttons, placeholders, svg's and audio/video tag to the html file
    # Note that the music pages are contained in one <div> that is initially not displayed (display:none)
    # The pages are displayed (display:block) one by one during play back
    g.write ('''
<body id="area" onload="setglobals();" onkeydown="keyDown (event);" style="margin:0px; height:100%">
<div id="buttons" style="height:5%">
<span style="float:right; position:relative; text-align:right; padding-right: 5px;">
  <label>enable sync</label>
  <input id="sync" type="checkbox" disabled="true" onclick="klikChk(event);"/>
  <div id="sync_out" style="display:none">
      <div id="event_display"></div>
      <button onclick="saveTiming();">save</button>
      <div id="help" style="text-align:left;"><span style="font-size:small;">?</span>
<table>
<tr><td colspan="2">You can synchronize by just clicking on the note that sounds. The tempo in all measures before
and after that note are updated by interpolation. Thus, a few sync points may be sufficient.
In addition you can use the following keys:
<tr><td><b>p</b><td>pause/continue
<tr><td><b>]</b>&nbsp;or&nbsp;<b>c</b><td>go to next measure
<tr><td><b>[</b>&nbsp;or&nbsp;<b>x</b><td>go to previous measure
<tr><td><b>,</b><td>shorten the duration of the current measure
<tr><td><b>.</b><td>lengthen the duration of the current measure
<tr><td><b>ctrl-,</b><td>shorten the initial offset (idle time before cursor starts moving).
<tr><td><b>ctrl-.</b><td>lengthen the initial offset.
<tr><td><b>w</b><td>save the timings to a text file
</table>
      </div>
  </div>
</span>
<input id="playbtn" type="button" value="play" onclick="play();" disabled="true"/>
<input id="nextbtn" type="button" value="next" onclick="next();" disabled="true"/>
<input id="prevbtn" type="button" value="prev" onclick="prev();" disabled="true"/>
measure: <input id="msrenum" type="text" size="2" onchange="gotoMsre();" disabled="true"/>
<span id="duur">please, wait for the score to load ...</span>&nbsp;&nbsp;
<span id="status"></span>&nbsp;&nbsp;
<span id="time"></span>
<span id="seekdial">Seeking to new position</span>
</div>''')
    # add the html5 audio tag (without controls, thus invisible) or videotag
    # or gernerate the html5 video tag
    if withVideo:    # video option given
        g.write ('''
<video id="sound" style="height:%d%%; margin-left:auto; margin-right:auto; display:block;"
    preload="auto"
    onseeking="seekpos(event);"
    onseeked="seekedpos(event);"
    onloadedmetadata="setDur(event);"
    onended="einde();"
    onprogress="loadAudio();"
    ontimeupdate="tick();">
  <source src="%s.webm" type="video/webm" />
  <source src="%s.mp4" type="video/mp4" />
  Your browser does not support the video element.
</video>''' % (videoHeight, fnm, fnm))
    else:            # generate audio
        g.write ('''
<audio id="sound"
    onseeking="seekpos(event);"
    onseeked="seekedpos(event);"
    onloadedmetadata="setDur(event);"
    onended="einde();"
    onprogress="loadAudio();"
    ontimeupdate="tick();">
  <source src="%s.ogg" type="audio/ogg" />
  <source src="%s.mp3" type="audio/mpeg" />
  Your browser does not support the audio element.
</audio>''' % (fnm, fnm))
    g.write ('''
<div id="music" style="display:none; overflow:auto; height:%d%%">
<div style="position:relative" onclick="klik(event);">
<div id="mark" style="background-color:blue; width:2px; height:0px; position:absolute; top:5px; left:5px"></div>
''' % (95 - videoHeight if withVideo else 95))
    for svg in svgs: # just copy all music pages into the html file
        for x in svg:
            g.write (x)
    g.write ('''</div>
</div>
<div id="saveDiv" style="display:none"></div>
</body>
</html>''')
    g.close ()
    print ofnm + ' written'

#----------------------
# start of main program
#----------------------
if __name__ == '__main__':
    parser = OptionParser (usage="%prog [-v] [-d OUTDIR] <inputfile.xhtml>", version='version %d' % VERSION)
    parser.add_option("-v", action="store_true", help="generate video element", default=False)
    parser.add_option ("-n", action="store_true", help="disable syncing", default=False)
    parser.add_option ("-s", action="store", type="int", help="set scroll margin to INT pixels", default=100, metavar='INT')
    parser.add_option ("-z", action="store", type="int", help="set video height (percentage)", default=40, metavar='INT')
    parser.add_option("-d", action="store", help="output to OUTDIR", metavar='OUTDIR', default='')
    options, args = parser.parse_args ()
    if len (args) != 1: parser.error ('no input file given')

    fnmext = args[0]
    fnm, ext = os.path.splitext (fnmext)
    if ext != '.xhtml': print 'input file should have extenion .xhtml\n%s' % parser.get_usage (); sys.exit ()
    if not os.path.exists (fnmext):
        print fnmext, 'does not exist.'; sys.exit ()
    genHtml (fnmext, options.v, options.n, options.d, options.s, options.z)
